@@ -23,6 +23,9 @@
 #include <map>
 #include <list>
 #include <vector>
+#ifdef HAVE_ATOMIC
+#include <atomic>
+#endif
 #include <unordered_map>
 #include <sstream>
 #include <regex>
@@ -31,6 +34,10 @@
 #include <sys/stat.h>
 #include <sys/socket.h>
 
+#define __FAVOR_BSD 1
+#include <netinet/tcp.h>
+#undef __FAVOR_BSD
+
 #include <errno.h>
 
 #include <arpa/inet.h>
@@ -50,207 +57,100 @@ using namespace std;
 #endif
 #include "nd-json.h"
 #include "nd-util.h"
+#include "nd-flow.h"
 
 // Enable flow hash cache debug logging
 //#define _ND_DEBUG_FHC 1
 
-#include "nd-flow.h"
-
 extern nd_global_config nd_config;
 extern nd_device_ethers device_ethers;
 
-ndFlowHashCache::ndFlowHashCache(const string &device, size_t cache_size)
-    : device(device), cache_size(cache_size) { }
-
-void ndFlowHashCache::push(const string &lower_hash, const string &upper_hash)
-{
-    nd_fhc_map::const_iterator i = lookup.find(lower_hash);
-
-    if (i != lookup.end()) {
-        nd_debug_printf("%s: WARNING: Found existing hash in flow hash cache on push.\n",
-            device.c_str());
-    }
-    else {
-        if (lookup.size() == cache_size) {
-//#if _ND_DEBUG_FHC
-            nd_debug_printf("%s: Purging old flow hash cache entries.\n", device.c_str());
-//#endif
-            for (size_t n = 0; n < cache_size / nd_config.fhc_purge_divisor; n++) {
-                pair<string, string> j = index.back();
-
-                nd_fhc_map::iterator k = lookup.find(j.first);
-                if (k == lookup.end()) {
-                    nd_debug_printf("%s: WARNING: flow hash cache index not found in map\n",
-                        device.c_str());
-                }
-                else
-                    lookup.erase(k);
-
-                index.pop_back();
-            }
-        }
-
-        index.push_front(make_pair(lower_hash, upper_hash));
-        lookup[lower_hash] = index.begin();
-#if _ND_DEBUG_FHC
-        nd_debug_printf("%s: Flow hash cache entries: %lu\n", device.c_str(), lookup.size());
-#endif
-    }
-}
-
-bool ndFlowHashCache::pop(const string &lower_hash, string &upper_hash)
-{
-    nd_fhc_map::iterator i = lookup.find(lower_hash);
-    if (i == lookup.end()) return false;
-
-    upper_hash = i->second->second;
-
-    index.erase(i->second);
-
-    index.push_front(make_pair(lower_hash, upper_hash));
-
-    i->second = index.begin();
-
-    return true;
-}
-
-void ndFlowHashCache::save(void)
-{
-    ostringstream os;
-
-    switch (nd_config.fhc_save) {
-    case ndFHC_PERSISTENT:
-        os << ND_PERSISTENT_STATEDIR << ND_FLOW_HC_FILE_NAME << device << ".dat";
-        break;
-    case ndFHC_VOLATILE:
-        os << ND_VOLATILE_STATEDIR << ND_FLOW_HC_FILE_NAME << device << ".dat";
-        break;
-    default:
-        return;
-    }
-
-    FILE *hf = fopen(os.str().c_str(), "wb");
-    if (hf == NULL) {
-        nd_printf("%s: WARNING: Error saving flow hash cache: %s: %s\n",
-            device.c_str(), os.str().c_str(), strerror(errno));
-        return;
-    }
-
-    nd_fhc_list::iterator i;
-    for (i = index.begin(); i != index.end(); i++) {
-        fwrite((*i).first.c_str(), 1, SHA1_DIGEST_LENGTH, hf);
-        fwrite((*i).second.c_str(), 1, SHA1_DIGEST_LENGTH, hf);
-    }
-    fclose(hf);
-
-    nd_debug_printf("%s: Saved %lu flow hash cache entries.\n",
-        device.c_str(), index.size ());
-}
-
-void ndFlowHashCache::load(void)
-{
-    ostringstream os;
-
-    switch (nd_config.fhc_save) {
-    case ndFHC_PERSISTENT:
-        os << ND_PERSISTENT_STATEDIR << ND_FLOW_HC_FILE_NAME << device << ".dat";
-        break;
-    case ndFHC_VOLATILE:
-        os << ND_VOLATILE_STATEDIR << ND_FLOW_HC_FILE_NAME << device << ".dat";
-        break;
-    default:
-        return;
-    }
-
-    FILE *hf = fopen(os.str().c_str(), "rb");
-    if (hf != NULL) {
-        do {
-            string digest_lower, digest_mdata;
-            uint8_t digest[SHA1_DIGEST_LENGTH * 2];
-
-            if (fread(digest, SHA1_DIGEST_LENGTH * 2, 1, hf) != 1) break;
-
-            digest_lower.assign((const char *)digest, SHA1_DIGEST_LENGTH);
-            digest_mdata.assign((const char *)&digest[SHA1_DIGEST_LENGTH],
-                SHA1_DIGEST_LENGTH);
-
-            push(digest_lower, digest_mdata);
-        }
-        while (! feof(hf));
-
-        fclose(hf);
-    }
-
-    nd_debug_printf("%s: Loaded %lu flow hash cache entries.\n",
-        device.c_str(), index.size());
-}
-
-ndFlow::ndFlow(bool internal)
-    : internal(internal), ip_version(0), ip_protocol(0), vlan_id(0),
-    ip_nat(false), tcp_fin(false),
+ndFlow::ndFlow(nd_ifaces::iterator iface)
+    : iface(iface), dpi_thread_id(-1), pkt(NULL),
+    ip_version(0), ip_protocol(0), vlan_id(0),
+    ts_first_seen(0), ts_first_update(0), ts_last_seen(0),
+    lower_map(LOWER_UNKNOWN), other_type(OTHER_UNKNOWN),
+    lower_mac{}, upper_mac{}, lower_addr{}, upper_addr{},
+    lower_addr4(NULL), lower_addr6(NULL), upper_addr4(NULL), upper_addr6(NULL),
+    lower_ip{}, upper_ip{},
+    lower_port(0), upper_port(0),
+    tunnel_type(TUNNEL_NONE), gtp{},
+    lower_bytes(0), upper_bytes(0), total_bytes(0),
+    lower_packets(0), upper_packets(0), total_packets(0),
+    detection_packets(0), detected_protocol{},
+    detected_protocol_name(NULL), detected_application_name(NULL),
+    ndpi_flow(NULL), id_src(NULL), id_dst(NULL),
+    digest_lower{}, digest_mdata{},
+    host_server_name{}, http{},
+    privacy_mask(0), origin(0), direction(0),
+    capture_filename{},
 #ifdef _ND_USE_CONNTRACK
     ct_id(0), ct_mark(0),
 #endif
-    ts_first_seen(0), ts_first_update(0), ts_last_seen(0),
-    lower_port(0), upper_port(0),
-    lower_bytes(0), upper_bytes(0), total_bytes(0),
-    lower_packets(0), upper_packets(0), total_packets(0),
-    detection_complete(false), detection_guessed(0),
-    ndpi_flow(NULL), id_src(NULL), id_dst(NULL),
-    privacy_mask(0), origin(0)
+#ifdef _ND_USE_NETLINK
+    lower_type(ndNETLINK_ATYPE_UNKNOWN), upper_type(ndNETLINK_ATYPE_UNKNOWN),
+#endif
+    flags{}
 {
-    memset(lower_mac, 0, ETH_ALEN);
-    memset(upper_mac, 0, ETH_ALEN);
-
-    memset(&lower_addr, 0, sizeof(struct sockaddr_storage));
-    memset(&upper_addr, 0, sizeof(struct sockaddr_storage));
-
     lower_addr4 = (struct sockaddr_in *)&lower_addr;
     lower_addr6 = (struct sockaddr_in6 *)&lower_addr;
     upper_addr4 = (struct sockaddr_in *)&upper_addr;
     upper_addr6 = (struct sockaddr_in6 *)&upper_addr;
 
-    memset(lower_ip, 0, INET6_ADDRSTRLEN);
-    memset(upper_ip, 0, INET6_ADDRSTRLEN);
+    gtp.version = 0xFF;
+}
 
-    memset(&detected_protocol, 0, sizeof(ndpi_protocol));
+ndFlow::ndFlow(const ndFlow &flow)
+    : iface(flow.iface), dpi_thread_id(-1), pkt(NULL),
+    ip_version(flow.ip_version), ip_protocol(flow.ip_protocol),
+    vlan_id(flow.vlan_id),
+    ts_first_seen(flow.ts_first_seen), ts_first_update(flow.ts_first_update),
+    ts_last_seen(flow.ts_last_seen),
+    lower_map(LOWER_UNKNOWN), other_type(OTHER_UNKNOWN),
+    lower_addr(flow.lower_addr), upper_addr(flow.upper_addr),
+    lower_ip{}, upper_ip{},
+    lower_port(flow.lower_port), upper_port(flow.upper_port),
+    tunnel_type(flow.tunnel_type), gtp(flow.gtp),
+    lower_bytes(0), upper_bytes(0), total_bytes(0),
+    lower_packets(0), upper_packets(0), total_packets(0),
+    detection_packets(0), detected_protocol{},
+    detected_protocol_name(NULL), detected_application_name(NULL),
+    ndpi_flow(NULL), id_src(NULL), id_dst(NULL),
+    host_server_name{}, http{},
+    privacy_mask(0), origin(0), direction(0),
+    capture_filename{},
+#ifdef _ND_USE_CONNTRACK
+    ct_id(0), ct_mark(0),
+#endif
+#ifdef _ND_USE_NETLINK
+    lower_type(ndNETLINK_ATYPE_UNKNOWN), upper_type(ndNETLINK_ATYPE_UNKNOWN),
+#endif
+    flags{}
+{
+    memcpy(lower_mac, flow.lower_mac, ETH_ALEN);
+    memcpy(upper_mac, flow.upper_mac, ETH_ALEN);
 
-    memset(digest_lower, 0, SHA1_DIGEST_LENGTH);
+    memcpy(digest_lower, flow.digest_lower, SHA1_DIGEST_LENGTH);
     memset(digest_mdata, 0, SHA1_DIGEST_LENGTH);
 
-    memset(host_server_name, 0, ND_MAX_HOSTNAME);
-
-    memset(http.user_agent, 0, ND_FLOW_UA_LEN);
-    memset(http.url, 0, ND_FLOW_URL_LEN);
-
-    memset(dhcp.fingerprint, 0, ND_FLOW_DHCPFP_LEN);
-    memset(dhcp.class_ident, 0, ND_FLOW_DHCPCI_LEN);
-
-    memset(ssh.client_agent, 0, ND_FLOW_SSH_UALEN);
-    memset(ssh.server_agent, 0, ND_FLOW_SSH_UALEN);
-
-    ssl.version = 0;
-    ssl.cipher_suite = 0;
-    memset(ssl.client_sni, 0, ND_FLOW_SSL_CNLEN);
-    memset(ssl.server_cn, 0, ND_FLOW_SSL_CNLEN);
-    memset(ssl.server_organization, 0, ND_FLOW_SSL_ORGLEN);
-    memset(ssl.client_ja3, 0, ND_FLOW_SSL_JA3LEN);
-    memset(ssl.server_ja3, 0, ND_FLOW_SSL_JA3LEN);
-    memset(ssl.cert_fingerprint, 0, ND_FLOW_SSL_HASH_LEN);
-
-    smtp.tls = false;
-
-    bt.info_hash_valid = 0;
-    memset(bt.info_hash, 0, ND_FLOW_BTIHASH_LEN);
-
-    memset(mdns.answer, 0, ND_FLOW_MDNS_ANSLEN);
-
-    memset(capture_filename, 0, sizeof(ND_FLOW_CAPTURE_TEMPLATE));
+    lower_addr4 = (struct sockaddr_in *)&lower_addr;
+    lower_addr6 = (struct sockaddr_in6 *)&lower_addr;
+    upper_addr4 = (struct sockaddr_in *)&upper_addr;
+    upper_addr6 = (struct sockaddr_in6 *)&upper_addr;
 }
 
 ndFlow::~ndFlow()
 {
     release();
+
+    if (detected_protocol_name != NULL) {
+        free(detected_protocol_name);
+        detected_protocol_name = NULL;
+    }
+    if (detected_application_name != NULL) {
+        free(detected_application_name);
+        detected_application_name = NULL;
+    }
 }
 
 void ndFlow::hash(const string &device,
@@ -265,8 +165,8 @@ void ndFlow::hash(const string &device,
     sha1_write(&ctx, (const char *)&ip_protocol, sizeof(ip_protocol));
     sha1_write(&ctx, (const char *)&vlan_id, sizeof(vlan_id));
 
-    sha1_write(&ctx, (const char *)&lower_mac, ETH_ALEN);
-    sha1_write(&ctx, (const char *)&upper_mac, ETH_ALEN);
+//    sha1_write(&ctx, (const char *)&lower_mac, ETH_ALEN);
+//    sha1_write(&ctx, (const char *)&upper_mac, ETH_ALEN);
 
     switch (ip_version) {
     case 4:
@@ -274,6 +174,13 @@ void ndFlow::hash(const string &device,
             (const char *)&lower_addr4->sin_addr, sizeof(struct in_addr));
         sha1_write(&ctx,
             (const char *)&upper_addr4->sin_addr, sizeof(struct in_addr));
+
+        if (lower_addr4->sin_addr.s_addr == 0 &&
+            upper_addr4->sin_addr.s_addr == 0xffffffff) {
+            // XXX: Hash in lower MAC for ethernet broadcasts (DHCPv4).
+            sha1_write(&ctx, (const char *)&lower_mac, ETH_ALEN);
+        }
+
         break;
     case 6:
         sha1_write(&ctx,
@@ -288,9 +195,13 @@ void ndFlow::hash(const string &device,
     sha1_write(&ctx, (const char *)&lower_port, sizeof(lower_port));
     sha1_write(&ctx, (const char *)&upper_port, sizeof(upper_port));
 
+//    nd_dprintf("hash: %s, %hhu, %hhu, %hu, [%hhx%hhx%hhx%hhx%hhx%hhx], [%hhx%hhx%hhx%hhx%hhx%hhx], %hhu, %hhu\n",
+//        device.c_str(), ip_version, ip_protocol, vlan_id,
+//        lower_mac[0], lower_mac[1], lower_mac[2], lower_mac[3], lower_mac[4], lower_mac[5],
+//        upper_mac[0], upper_mac[1], upper_mac[2], upper_mac[3], upper_mac[4], upper_mac[5],
+//        lower_port, upper_port);
+
     if (hash_mdata) {
-        sha1_write(&ctx,
-            (const char *)&detection_guessed, sizeof(detection_guessed));
         sha1_write(&ctx,
             (const char *)&detected_protocol, sizeof(ndpi_protocol));
 
@@ -300,11 +211,11 @@ void ndFlow::hash(const string &device,
         }
         if (has_ssl_client_sni()) {
             sha1_write(&ctx,
-                ssl.client_sni, strnlen(ssl.client_sni, ND_FLOW_SSL_CNLEN));
+                ssl.client_sni, strnlen(ssl.client_sni, ND_FLOW_TLS_CNLEN));
         }
         if (has_ssl_server_cn()) {
             sha1_write(&ctx,
-                ssl.server_cn, strnlen(ssl.server_cn, ND_FLOW_SSL_CNLEN));
+                ssl.server_cn, strnlen(ssl.server_cn, ND_FLOW_TLS_CNLEN));
         }
         if (has_bt_info_hash()) {
             sha1_write(&ctx, bt.info_hash, ND_FLOW_BTIHASH_LEN);
@@ -350,7 +261,7 @@ int ndFlow::dump(pcap_t *pcap, const uint8_t *digest)
     pcap_dumper_t *pcap_dumper = pcap_dump_open(pcap, capture_filename);
 
     if (pcap_dumper == NULL) {
-        nd_debug_printf("%s: pcap_dump_open: %s: %s\n",
+        nd_dprintf("%s: pcap_dump_open: %s: %s\n",
             __PRETTY_FUNCTION__, capture_filename, "unknown");
         return -1;
     }
@@ -374,6 +285,7 @@ void ndFlow::reset(void)
 
 void ndFlow::release(void)
 {
+    if (pkt != NULL) { delete [] pkt; pkt = NULL; }
     if (ndpi_flow != NULL) { ndpi_free_flow(ndpi_flow); ndpi_flow = NULL; }
     if (id_src != NULL) { delete id_src; id_src = NULL; }
     if (id_dst != NULL) { delete id_dst; id_dst = NULL; }
@@ -542,9 +454,8 @@ bool ndFlow::has_ssdp_headers(void)
     );
 }
 
-void ndFlow::print(const char *tag, struct ndpi_detection_module_struct *ndpi)
+void ndFlow::print(void)
 {
-    char *p = NULL, buffer[64];
     const char *lower_name = lower_ip, *upper_name = upper_ip;
 
     if (ND_DEBUG_WITH_ETHERS) {
@@ -564,30 +475,27 @@ void ndFlow::print(const char *tag, struct ndpi_detection_module_struct *ndpi)
             upper_name = i->second.c_str();
     }
 
-    if (detected_protocol.app_protocol) {
-        ndpi_protocol2name(ndpi,
-            detected_protocol, buffer, sizeof(buffer));
-        p = buffer;
-    }
-    else
-        p = ndpi_get_proto_name(ndpi, detected_protocol.master_protocol);
+    string iface_name;
+    nd_iface_name(iface->second, iface_name);
 
     string digest;
     nd_sha1_to_string((const uint8_t *)bt.info_hash, digest);
 
     nd_flow_printf(
-        "%s: [%c%c%c%c%c%c] %s %s:%hu %c%c%c %s:%hu%s%s%s%s%s%s%s%s%s\n",
-        tag,
-        (internal) ? 'i' : 'e',
+        "%s: [%c%c%c%c%c%c] %s%s%s %s:%hu %c%c%c %s:%hu%s%s%s%s%s%s%s%s%s\n",
+        iface_name.c_str(),
+        (iface->first) ? 'i' : 'e',
         (ip_version == 4) ? '4' : (ip_version == 6) ? '6' : '-',
-        ip_nat ? 'n' : '-',
-        (detection_guessed & ND_FLOW_GUESS_PROTO) ? 'g' : '-',
-        (dhc_hit) ? 'd' : '-',
+        flags.ip_nat ? 'n' : '-',
+        (flags.detection_guessed) ? 'g' : '-',
+        (flags.dhc_hit) ? 'd' : '-',
         (privacy_mask & PRIVATE_LOWER) ? 'p' :
             (privacy_mask & PRIVATE_UPPER) ? 'P' :
             (privacy_mask & (PRIVATE_LOWER | PRIVATE_UPPER)) ? 'X' :
             '-',
-        p,
+        detected_protocol_name,
+        (detected_application_name != NULL) ? "." : "",
+        (detected_application_name != NULL) ? detected_application_name : "",
         lower_name, ntohs(lower_port),
         (origin == ORIGIN_LOWER || origin == ORIGIN_UNKNOWN) ? '-' : '<',
         (origin == ORIGIN_UNKNOWN) ? '?' : '-',
@@ -607,149 +515,74 @@ void ndFlow::print(const char *tag, struct ndpi_detection_module_struct *ndpi)
 
     if (ND_DEBUG &&
         detected_protocol.master_protocol == NDPI_PROTOCOL_SSL &&
-        ! (detection_guessed & ND_FLOW_GUESS_PROTO) && ssl.version == 0x0000) {
-        nd_debug_printf("%s: SSL with no SSL/TLS verison.\n", tag);
+        flags.detection_guessed == false && ssl.version == 0x0000) {
+        nd_dprintf("%s: SSL with no SSL/TLS verison.\n", iface->second.c_str());
     }
 }
 
-void ndFlow::json_encode(json &j,
-    struct ndpi_detection_module_struct *ndpi, bool include_stats)
+void ndFlow::update_lower_maps(void)
 {
-    char mac_addr[ND_STR_ETHALEN + 1];
-    string other_type = "unknown";
-    string _lower_mac = "local_mac", _upper_mac = "other_mac";
-    string _lower_ip = "local_ip", _upper_ip = "other_ip";
-    string _lower_port = "local_port", _upper_port = "other_port";
-    string _lower_bytes = "local_bytes", _upper_bytes = "other_bytes";
-    string _lower_packets = "local_packets", _upper_packets = "other_packets";
+    if (lower_map == LOWER_UNKNOWN)
+        get_lower_map(lower_type, upper_type, lower_map, other_type);
 
-    string digest;
-    nd_sha1_to_string(digest_mdata, digest);
+    switch (tunnel_type) {
+    case TUNNEL_GTP:
+        if (gtp.lower_map == LOWER_UNKNOWN) {
+            get_lower_map(
+                gtp.lower_type, gtp.upper_type, gtp.lower_map, gtp.other_type
+            );
+        }
+    break;
+    }
+}
 
-    j["digest"] = digest;
-    j["ip_nat"] = ip_nat;
-    j["dhc_hit"] = dhc_hit;
-#ifdef _ND_USE_CONNTRACK
-    j["ct_id"] = ct_id;
-    j["ct_mark"] = ct_mark;
+void ndFlow::get_lower_map(
+#ifdef _ND_USE_NETLINK
+    ndNetlinkAddressType lt,
+    ndNetlinkAddressType ut,
 #endif
-    j["ip_version"] = (unsigned)ip_version;
-    j["ip_protocol"] = (unsigned)ip_protocol;
-    j["vlan_id"] = (unsigned)vlan_id;
-#ifndef _ND_USE_NETLINK
-    other_type = "unsupported";
-#else
-    if (lower_type == ndNETLINK_ATYPE_ERROR ||
-        upper_type == ndNETLINK_ATYPE_ERROR) {
-        other_type = "error";
+    uint8_t &lm, uint8_t &ot)
+{
+    if (lt == ndNETLINK_ATYPE_ERROR ||
+        ut == ndNETLINK_ATYPE_ERROR) {
+        ot = OTHER_ERROR;
+        return;
     }
-    else if (lower_type == ndNETLINK_ATYPE_LOCALIP &&
-        upper_type == ndNETLINK_ATYPE_LOCALNET) {
-        other_type = "local";
-        _lower_mac = "other_mac";
-        _lower_ip = "other_ip";
-        _lower_port = "other_port";
-        _lower_bytes = "other_bytes";
-        _lower_packets = "other_packets";
-        _upper_mac = "local_mac";
-        _upper_ip = "local_ip";
-        _upper_port = "local_port";
-        _upper_bytes = "local_bytes";
-        _upper_packets = "local_packets";
+    else if (lt == ndNETLINK_ATYPE_LOCALIP &&
+        ut == ndNETLINK_ATYPE_LOCALNET) {
+        lm = LOWER_OTHER;
+        ot = OTHER_LOCAL;
     }
-    else if (lower_type == ndNETLINK_ATYPE_LOCALNET &&
-        upper_type == ndNETLINK_ATYPE_LOCALIP) {
-        other_type = "local";
-        _lower_mac = "local_mac";
-        _lower_ip = "local_ip";
-        _lower_port = "local_port";
-        _lower_bytes = "local_bytes";
-        _lower_packets = "local_packets";
-        _upper_mac = "other_mac";
-        _upper_ip = "other_ip";
-        _upper_port = "other_port";
-        _upper_bytes = "other_bytes";
-        _upper_packets = "other_packets";
+    else if (lt == ndNETLINK_ATYPE_LOCALNET &&
+        ut == ndNETLINK_ATYPE_LOCALIP) {
+        lm = LOWER_LOCAL;
+        ot = OTHER_LOCAL;
     }
-    else if (lower_type == ndNETLINK_ATYPE_MULTICAST) {
-        other_type = "multicast";
-        _lower_mac = "other_mac";
-        _lower_ip = "other_ip";
-        _lower_port = "other_port";
-        _lower_bytes = "other_bytes";
-        _lower_packets = "other_packets";
-        _upper_mac = "local_mac";
-        _upper_ip = "local_ip";
-        _upper_port = "local_port";
-        _upper_bytes = "local_bytes";
-        _upper_packets = "local_packets";
+    else if (lt == ndNETLINK_ATYPE_MULTICAST) {
+        lm = LOWER_OTHER;
+        ot = OTHER_MULTICAST;
     }
-    else if (upper_type == ndNETLINK_ATYPE_MULTICAST) {
-        other_type = "multicast";
-        _lower_mac = "local_mac";
-        _lower_ip = "local_ip";
-        _lower_port = "local_port";
-        _lower_bytes = "local_bytes";
-        _lower_packets = "local_packets";
-        _upper_mac = "other_mac";
-        _upper_ip = "other_ip";
-        _upper_port = "other_port";
-        _upper_bytes = "other_bytes";
-        _upper_packets = "other_packets";
+    else if (ut == ndNETLINK_ATYPE_MULTICAST) {
+        lm = LOWER_LOCAL;
+        ot = OTHER_MULTICAST;
     }
-    else if (lower_type == ndNETLINK_ATYPE_BROADCAST) {
-        other_type = "broadcast";
-        _lower_mac = "other_mac";
-        _lower_ip = "other_ip";
-        _lower_port = "other_port";
-        _lower_bytes = "other_bytes";
-        _lower_packets = "other_packets";
-        _upper_mac = "local_mac";
-        _upper_ip = "local_ip";
-        _upper_port = "local_port";
-        _upper_bytes = "local_bytes";
-        _upper_packets = "local_packets";
+    else if (lt == ndNETLINK_ATYPE_BROADCAST) {
+        lm = LOWER_OTHER;
+        ot = OTHER_BROADCAST;
     }
-    else if (upper_type == ndNETLINK_ATYPE_BROADCAST) {
-        other_type = "broadcast";
-        _lower_mac = "local_mac";
-        _lower_ip = "local_ip";
-        _lower_port = "local_port";
-        _lower_bytes = "local_bytes";
-        _lower_packets = "local_packets";
-        _upper_mac = "other_mac";
-        _upper_ip = "other_ip";
-        _upper_port = "other_port";
-        _upper_bytes = "other_bytes";
-        _upper_packets = "other_packets";
+    else if (ut == ndNETLINK_ATYPE_BROADCAST) {
+        lm = LOWER_LOCAL;
+        ot = OTHER_BROADCAST;
     }
-    else if (lower_type == ndNETLINK_ATYPE_PRIVATE &&
-        upper_type == ndNETLINK_ATYPE_LOCALNET) {
-        other_type = "local";
-        _lower_mac = "other_mac";
-        _lower_ip = "other_ip";
-        _lower_port = "other_port";
-        _lower_bytes = "other_bytes";
-        _lower_packets = "other_packets";
-        _upper_mac = "local_mac";
-        _upper_ip = "local_ip";
-        _upper_port = "local_port";
-        _upper_bytes = "local_bytes";
-        _upper_packets = "local_packets";
+    else if (lt == ndNETLINK_ATYPE_PRIVATE &&
+        ut == ndNETLINK_ATYPE_LOCALNET) {
+        lm = LOWER_OTHER;
+        ot = OTHER_LOCAL;
     }
-    else if (lower_type == ndNETLINK_ATYPE_LOCALNET &&
-        upper_type == ndNETLINK_ATYPE_PRIVATE) {
-        other_type = "local";
-        _lower_mac = "local_mac";
-        _lower_ip = "local_ip";
-        _lower_port = "local_port";
-        _lower_bytes = "local_bytes";
-        _lower_packets = "local_packets";
-        _upper_mac = "other_mac";
-        _upper_ip = "other_ip";
-        _upper_port = "other_port";
-        _upper_bytes = "other_bytes";
-        _upper_packets = "other_packets";
+    else if (lt == ndNETLINK_ATYPE_LOCALNET &&
+        ut == ndNETLINK_ATYPE_PRIVATE) {
+        lm = LOWER_LOCAL;
+        ot = OTHER_LOCAL;
     }
 #if 0
     // TODO: Further investigation required!
@@ -759,9 +592,63 @@ void ndFlow::json_encode(json &j,
     // deployment (gateway/port mirror modes).
 #endif
     else if (ip_version != 6 &&
-        lower_type == ndNETLINK_ATYPE_PRIVATE &&
-        upper_type == ndNETLINK_ATYPE_PRIVATE) {
-        other_type = "local";
+        lt == ndNETLINK_ATYPE_PRIVATE &&
+        ut == ndNETLINK_ATYPE_PRIVATE) {
+        lm = LOWER_LOCAL;
+        ot = OTHER_LOCAL;
+    }
+    else if (lt == ndNETLINK_ATYPE_PRIVATE &&
+        ut == ndNETLINK_ATYPE_LOCALIP) {
+        lm = LOWER_OTHER;
+        ot = OTHER_REMOTE;
+    }
+    else if (lt == ndNETLINK_ATYPE_LOCALIP &&
+        ut == ndNETLINK_ATYPE_PRIVATE) {
+        lm = LOWER_LOCAL;
+        ot = OTHER_REMOTE;
+    }
+    else if (lt == ndNETLINK_ATYPE_LOCALNET &&
+        ut == ndNETLINK_ATYPE_LOCALNET) {
+        lm = LOWER_LOCAL;
+        ot = OTHER_LOCAL;
+    }
+    else if (lt == ndNETLINK_ATYPE_UNKNOWN) {
+        lm = LOWER_OTHER;
+        ot = OTHER_REMOTE;
+    }
+    else if (ut == ndNETLINK_ATYPE_UNKNOWN) {
+        lm = LOWER_LOCAL;
+        ot = OTHER_REMOTE;
+    }
+}
+
+void ndFlow::json_encode(json &j, uint8_t encode_includes)
+{
+    char mac_addr[ND_STR_ETHALEN + 1];
+    string _other_type = "unknown";
+    string _lower_mac = "local_mac", _upper_mac = "other_mac";
+    string _lower_ip = "local_ip", _upper_ip = "other_ip";
+    string _lower_gtp_ip = "local_ip", _upper_gtp_ip = "other_ip";
+    string _lower_port = "local_port", _upper_port = "other_port";
+    string _lower_gtp_port = "local_port", _upper_gtp_port = "other_port";
+    string _lower_bytes = "local_bytes", _upper_bytes = "other_bytes";
+    string _lower_packets = "local_packets", _upper_packets = "other_packets";
+
+    string digest;
+    uint8_t digest_null[SHA1_DIGEST_LENGTH] = { '\0' };
+
+    if (memcmp(digest_mdata, digest_null, SHA1_DIGEST_LENGTH) != 0) {
+        nd_sha1_to_string(digest_mdata, digest);
+        j["digest"] = digest;
+    } else {
+        nd_sha1_to_string(digest_lower, digest);
+        j["digest"] = digest;
+    }
+
+    j["last_seen_at"] = ts_last_seen;
+
+    switch (lower_map) {
+    case LOWER_LOCAL:
         _lower_mac = "local_mac";
         _lower_ip = "local_ip";
         _lower_port = "local_port";
@@ -772,10 +659,8 @@ void ndFlow::json_encode(json &j,
         _upper_port = "other_port";
         _upper_bytes = "other_bytes";
         _upper_packets = "other_packets";
-    }
-    else if (lower_type == ndNETLINK_ATYPE_PRIVATE &&
-        upper_type == ndNETLINK_ATYPE_LOCALIP) {
-        other_type = "remote";
+        break;
+    case LOWER_OTHER:
         _lower_mac = "other_mac";
         _lower_ip = "other_ip";
         _lower_port = "other_port";
@@ -786,230 +671,263 @@ void ndFlow::json_encode(json &j,
         _upper_port = "local_port";
         _upper_bytes = "local_bytes";
         _upper_packets = "local_packets";
+        break;
     }
-    else if (lower_type == ndNETLINK_ATYPE_LOCALIP &&
-        upper_type == ndNETLINK_ATYPE_PRIVATE) {
-        other_type = "remote";
-        _lower_mac = "local_mac";
-        _lower_ip = "local_ip";
-        _lower_port = "local_port";
-        _lower_bytes = "local_bytes";
-        _lower_packets = "local_packets";
-        _upper_mac = "other_mac";
-        _upper_ip = "other_ip";
-        _upper_port = "other_port";
-        _upper_bytes = "other_bytes";
-        _upper_packets = "other_packets";
-    }
-    else if (lower_type == ndNETLINK_ATYPE_LOCALNET &&
-        upper_type == ndNETLINK_ATYPE_LOCALNET) {
-        other_type = "local";
-        _lower_mac = "local_mac";
-        _lower_ip = "local_ip";
-        _lower_port = "local_port";
-        _lower_bytes = "local_bytes";
-        _lower_packets = "local_packets";
-        _upper_mac = "other_mac";
-        _upper_ip = "other_ip";
-        _upper_port = "other_port";
-        _upper_bytes = "other_bytes";
-        _upper_packets = "other_packets";
-    }
-    else if (lower_type == ndNETLINK_ATYPE_UNKNOWN) {
-        other_type = "remote";
-        _lower_mac = "other_mac";
-        _lower_ip = "other_ip";
-        _lower_port = "other_port";
-        _lower_bytes = "other_bytes";
-        _lower_packets = "other_packets";
-        _upper_mac = "local_mac";
-        _upper_ip = "local_ip";
-        _upper_port = "local_port";
-        _upper_bytes = "local_bytes";
-        _upper_packets = "local_packets";
-    }
-    else if (upper_type == ndNETLINK_ATYPE_UNKNOWN) {
-        other_type = "remote";
-        _lower_mac = "local_mac";
-        _lower_ip = "local_ip";
-        _lower_port = "local_port";
-        _lower_bytes = "local_bytes";
-        _lower_packets = "local_packets";
-        _upper_mac = "other_mac";
-        _upper_ip = "other_ip";
-        _upper_port = "other_port";
-        _upper_bytes = "other_bytes";
-        _upper_packets = "other_packets";
+
+    switch (other_type) {
+    case OTHER_LOCAL:
+        _other_type = "local";
+        break;
+    case OTHER_MULTICAST:
+        _other_type = "multicast";
+        break;
+    case OTHER_BROADCAST:
+        _other_type = "broadcast";
+        break;
+    case OTHER_REMOTE:
+        _other_type = "remote";
+        break;
+    case OTHER_UNSUPPORTED:
+        _other_type = "unsupported";
+        break;
+    case OTHER_ERROR:
+        _other_type = "error";
+        break;
     }
+
+    if (encode_includes & ENCODE_METADATA) {
+        j["ip_nat"] = (bool)flags.ip_nat;
+        j["dhc_hit"] = (bool)flags.dhc_hit;
+#ifdef _ND_USE_CONNTRACK
+        j["ct_id"] = ct_id;
+        j["ct_mark"] = ct_mark;
+#endif
+        j["ip_version"] = (unsigned)ip_version;
+        j["ip_protocol"] = (unsigned)ip_protocol;
+        j["vlan_id"] = (unsigned)vlan_id;
 #ifndef _ND_LEAN_AND_MEAN
-    // 10.110.80.1: address is: PRIVATE
-    // 67.204.229.236: address is: LOCALIP
-    if (ND_DEBUG && other_type == "unknown") {
-        ndNetlink::PrintType(lower_ip, lower_type);
-        ndNetlink::PrintType(upper_ip, upper_type);
-        //exit(1);
-    }
+        // 10.110.80.1: address is: PRIVATE
+        // 67.204.229.236: address is: LOCALIP
+        if (ND_DEBUG && _other_type == "unknown") {
+            ndNetlink::PrintType(lower_ip, lower_type);
+            ndNetlink::PrintType(upper_ip, upper_type);
+            //exit(1);
+        }
 #endif
-#endif
-    j["other_type"] = other_type;
+        j["other_type"] = _other_type;
 
-    switch (origin) {
-    case ORIGIN_UPPER:
-        j["local_origin"] =
-            (_lower_ip == "local_ip") ? false : true;
-        break;
-    case ORIGIN_LOWER:
-    default:
-        j["local_origin"] =
-            (_lower_ip == "local_ip") ? true : false;
-        break;
-    }
+        switch (origin) {
+        case ORIGIN_UPPER:
+            j["local_origin"] =
+                (_lower_ip == "local_ip") ? false : true;
+            break;
+        case ORIGIN_LOWER:
+        default:
+            j["local_origin"] =
+                (_lower_ip == "local_ip") ? true : false;
+            break;
+        }
 
-    // 00-52-14 to 00-52-FF: Unassigned (small allocations)
-    if (privacy_mask & PRIVATE_LOWER)
-        snprintf(mac_addr, sizeof(mac_addr), "00:52:14:00:00:00");
-    else {
-        snprintf(mac_addr, sizeof(mac_addr),
-            "%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx",
-            lower_mac[0], lower_mac[1], lower_mac[2],
-            lower_mac[3], lower_mac[4], lower_mac[5]
-        );
-    }
-    j[_lower_mac] = mac_addr;
+        // 00-52-14 to 00-52-FF: Unassigned (small allocations)
+        if (privacy_mask & PRIVATE_LOWER)
+            snprintf(mac_addr, sizeof(mac_addr), "00:52:14:00:00:00");
+        else {
+            snprintf(mac_addr, sizeof(mac_addr),
+                "%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx",
+                lower_mac[0], lower_mac[1], lower_mac[2],
+                lower_mac[3], lower_mac[4], lower_mac[5]
+            );
+        }
+        j[_lower_mac] = mac_addr;
 
-    if (privacy_mask & PRIVATE_UPPER)
-        snprintf(mac_addr, sizeof(mac_addr), "00:52:FF:00:00:00");
-    else {
-        snprintf(mac_addr, sizeof(mac_addr),
-            "%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx",
-            upper_mac[0], upper_mac[1], upper_mac[2],
-            upper_mac[3], upper_mac[4], upper_mac[5]
-        );
-    }
-    j[_upper_mac] = mac_addr;
+        if (privacy_mask & PRIVATE_UPPER)
+            snprintf(mac_addr, sizeof(mac_addr), "00:52:FF:00:00:00");
+        else {
+            snprintf(mac_addr, sizeof(mac_addr),
+                "%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx",
+                upper_mac[0], upper_mac[1], upper_mac[2],
+                upper_mac[3], upper_mac[4], upper_mac[5]
+            );
+        }
+        j[_upper_mac] = mac_addr;
 
-    if (privacy_mask & PRIVATE_LOWER) {
-        if (ip_version == 4)
-            j[_lower_ip] = ND_PRIVATE_IPV4 "253";
+        if (privacy_mask & PRIVATE_LOWER) {
+            if (ip_version == 4)
+                j[_lower_ip] = ND_PRIVATE_IPV4 "253";
+            else
+                j[_lower_ip] = ND_PRIVATE_IPV6 "fd";
+        }
         else
-            j[_lower_ip] = ND_PRIVATE_IPV6 "fd";
-    }
-    else
-        j[_lower_ip] = lower_ip;
+            j[_lower_ip] = lower_ip;
 
-    if (privacy_mask & PRIVATE_UPPER) {
-        if (ip_version == 4)
-            j[_upper_ip] = ND_PRIVATE_IPV4 "254";
+        if (privacy_mask & PRIVATE_UPPER) {
+            if (ip_version == 4)
+                j[_upper_ip] = ND_PRIVATE_IPV4 "254";
+            else
+                j[_upper_ip] = ND_PRIVATE_IPV6 "fe";
+        }
         else
-            j[_upper_ip] = ND_PRIVATE_IPV6 "fe";
+            j[_upper_ip] = upper_ip;
+
+        j[_lower_port] = (unsigned)ntohs(lower_port);
+        j[_upper_port] = (unsigned)ntohs(upper_port);
+
+        j["detected_protocol"] =
+            (unsigned)detected_protocol.master_protocol;
+        j["detected_protocol_name"] =
+            (detected_protocol_name != NULL) ? detected_protocol_name : "Unknown";
+
+        j["detected_application"] =
+            (unsigned)detected_protocol.app_protocol;
+        j["detected_application_name"] =
+            (detected_application_name != NULL) ? detected_application_name : "Unknown";
+
+        j["detection_guessed"] = (unsigned)flags.detection_guessed;
+
+        if (host_server_name[0] != '\0')
+            j["host_server_name"] = host_server_name;
+
+        if (has_http_user_agent() || has_http_url()) {
+
+            if (has_http_user_agent())
+                j["http"]["user_agent"] = http.user_agent;
+            if (has_http_url())
+                j["http"]["url"] = http.url;
+        }
+
+        if (has_dhcp_fingerprint() || has_dhcp_class_ident()) {
+
+            if (has_dhcp_fingerprint())
+                j["dhcp"]["fingerprint"] = dhcp.fingerprint;
+
+            if (has_dhcp_class_ident())
+                j["dhcp"]["class_ident"] = dhcp.class_ident;
+        }
+
+        if (has_ssh_client_agent() || has_ssh_server_agent()) {
+
+            if (has_ssh_client_agent())
+                j["ssh"]["client"] = ssh.client_agent;
+
+            if (has_ssh_server_agent())
+                j["ssh"]["server"] = ssh.server_agent;
+        }
+
+        if (has_ssl_client_sni() || has_ssl_server_cn()) {
+
+            char tohex[7];
+
+            sprintf(tohex, "0x%04hx", ssl.version);
+            j["ssl"]["version"] = tohex;
+
+            sprintf(tohex, "0x%04hx", ssl.cipher_suite);
+            j["ssl"]["cipher_suite"] = tohex;
+
+            if (has_ssl_client_sni())
+                j["ssl"]["client_sni"] = ssl.client_sni;
+
+            if (has_ssl_server_cn())
+                j["ssl"]["server_cn"] = ssl.server_cn;
+
+            if (has_ssl_server_organization())
+                j["ssl"]["organization"] = ssl.server_organization;
+
+            if (has_ssl_client_ja3())
+                j["ssl"]["client_ja3"] = ssl.client_ja3;
+
+            if (has_ssl_server_ja3())
+                j["ssl"]["server_ja3"] = ssl.server_ja3;
+
+            if (ssl.cert_fingerprint_found) {
+                nd_sha1_to_string((const uint8_t *)ssl.cert_fingerprint, digest);
+                j["ssl"]["fingerprint"] = digest;
+            }
+        }
+
+        if (has_bt_info_hash()) {
+
+            nd_sha1_to_string((const uint8_t *)bt.info_hash, digest);
+            j["bt"]["info_hash"] = digest;
+        }
+
+        if (has_mdns_answer()) {
+
+            j["mdns"]["answer"] = mdns.answer;
+        }
+
+        if (has_ssdp_headers()) {
+
+            j["ssdp"] = ssdp.headers;
+
+        }
+
+        j["first_seen_at"] = ts_first_seen;
+        j["first_update_at"] = ts_first_update;
     }
-    else
-        j[_upper_ip] = upper_ip;
 
-    j[_lower_port] = (unsigned)ntohs(lower_port);
-    j[_upper_port] = (unsigned)ntohs(upper_port);
+    if (encode_includes & ENCODE_TUNNELS) {
+        string _lower_teid = "local_teid", _upper_teid = "other_teid";
 
-    if (include_stats) {
+        switch (tunnel_type) {
+        case TUNNEL_GTP:
+            switch (gtp.lower_map) {
+            case LOWER_LOCAL:
+                _lower_ip = "local_ip";
+                _lower_port = "local_port";
+                _lower_teid = "local_teid";
+                _upper_ip = "other_ip";
+                _upper_port = "other_port";
+                _upper_teid = "other_teid";
+                break;
+            case LOWER_OTHER:
+                _lower_ip = "other_ip";
+                _lower_port = "other_port";
+                _lower_teid = "other_teid";
+                _upper_ip = "local_ip";
+                _upper_port = "local_port";
+                _upper_teid = "local_teid";
+                break;
+            }
+
+            switch (gtp.other_type) {
+            case OTHER_LOCAL:
+                _other_type = "local";
+                break;
+            case OTHER_REMOTE:
+                _other_type = "remote";
+                break;
+            case OTHER_ERROR:
+                _other_type = "error";
+                break;
+            case OTHER_UNSUPPORTED:
+            default:
+                _other_type = "unsupported";
+                break;
+            }
+
+            j["gtp"]["version"] = gtp.version;
+            j["gtp"]["ip_version"] = gtp.ip_version;
+            j["gtp"][_lower_ip] = gtp.lower_ip;
+            j["gtp"][_upper_ip] = gtp.upper_ip;
+            j["gtp"][_lower_port] = (unsigned)htons(gtp.lower_port);
+            j["gtp"][_upper_port] = (unsigned)htons(gtp.upper_port);
+            j["gtp"][_lower_teid] = htonl(gtp.lower_teid);
+            j["gtp"][_upper_teid] = htonl(gtp.upper_teid);
+            j["gtp"]["other_type"] = _other_type;
+
+            break;
+        }
+    }
+
+    if (encode_includes & ENCODE_STATS) {
         j[_lower_bytes] = lower_bytes;
         j[_upper_bytes] = upper_bytes;
         j[_lower_packets] = lower_packets;
         j[_upper_packets] = upper_packets;
         j["total_packets"] = total_packets;
         j["total_bytes"] = total_bytes;
+        j["detection_packets"] = detection_packets;
     }
-
-    j["detected_protocol"] =
-        (unsigned)detected_protocol.master_protocol;
-    j["detected_protocol_name"] =
-        ndpi_get_proto_name(ndpi, detected_protocol.master_protocol);
-
-    j["detected_application"] =
-        (unsigned)detected_protocol.app_protocol;
-    j["detected_application_name"] =
-        ndpi_get_proto_name(ndpi, detected_protocol.app_protocol);
-
-    j["detection_guessed"] = detection_guessed;
-
-    if (host_server_name[0] != '\0')
-        j["host_server_name"] = host_server_name;
-
-    if (has_http_user_agent() || has_http_url()) {
-
-        if (has_http_user_agent())
-            j["http"]["user_agent"] = http.user_agent;
-        if (has_http_url())
-            j["http"]["url"] = http.url;
-    }
-
-    if (has_dhcp_fingerprint() || has_dhcp_class_ident()) {
-
-        if (has_dhcp_fingerprint())
-            j["dhcp"]["fingerprint"] = dhcp.fingerprint;
-
-        if (has_dhcp_class_ident())
-            j["dhcp"]["class_ident"] = dhcp.class_ident;
-    }
-
-    if (has_ssh_client_agent() || has_ssh_server_agent()) {
-
-        if (has_ssh_client_agent())
-            j["ssh"]["client"] = ssh.client_agent;
-
-        if (has_ssh_server_agent())
-            j["ssh"]["server"] = ssh.server_agent;
-    }
-
-    if (has_ssl_client_sni() || has_ssl_server_cn()) {
-
-        char tohex[7];
-
-        sprintf(tohex, "0x%04hx", ssl.version);
-        j["ssl"]["version"] = tohex;
-
-        sprintf(tohex, "0x%04hx", ssl.cipher_suite);
-        j["ssl"]["cipher_suite"] = tohex;
-
-        if (has_ssl_client_sni())
-            j["ssl"]["client_sni"] = ssl.client_sni;
-
-        if (has_ssl_server_cn())
-            j["ssl"]["server_cn"] = ssl.server_cn;
-
-        if (has_ssl_server_organization())
-            j["ssl"]["organization"] = ssl.server_organization;
-
-        if (has_ssl_client_ja3())
-            j["ssl"]["client_ja3"] = ssl.client_ja3;
-
-        if (has_ssl_server_ja3())
-            j["ssl"]["server_ja3"] = ssl.server_ja3;
-
-        if (ssl.cert_fingerprint_found) {
-            nd_sha1_to_string((const uint8_t *)ssl.cert_fingerprint, digest);
-            j["ssl"]["fingerprint"] = digest;
-        }
-    }
-
-    if (has_bt_info_hash()) {
-
-        nd_sha1_to_string((const uint8_t *)bt.info_hash, digest);
-        j["bt"]["info_hash"] = digest;
-    }
-
-    if (has_mdns_answer()) {
-
-        j["mdns"]["answer"] = mdns.answer;
-    }
-
-    if (has_ssdp_headers()) {
-
-        j["ssdp"] = ssdp.headers;
-
-    }
-
-    j["first_seen_at"] = ts_first_seen;
-    j["first_update_at"] = ts_first_update;
-    j["last_seen_at"] = ts_last_seen;
 }
 
 // vi: expandtab shiftwidth=4 softtabstop=4 tabstop=4
